# Generated by Django 4.1.1 on 2022-11-09 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0013_auto_20220822_0754'),
    ]

    operations = [
        migrations.AlterField(
            model_name='qrcode',
            name='title',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
    ]



# Generated by Django 3.0.11 on 2021-04-14 14:50

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('qrtoolkit_core', '0002_auto_20210202_1257'),
    ]

    operations = [
        migrations.AddField(
            model_name='department',
            name='users',
            field=models.ManyToManyField(related_name='departments', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.0.11 on 2022-08-22 07:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0012_qrcode_kiosk_introduction'),
    ]

    operations = [
        migrations.AlterField(
            model_name='qrcode',
            name='kiosk_introduction',
            field=models.TextField(blank=True, default='', null=True),
        ),
    ]



# Generated by Django 3.0.11 on 2021-09-29 16:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0009_linkurl_visible'),
    ]

    operations = [
        migrations.AlterField(
            model_name='linkurl',
            name='visible',
            field=models.BooleanField(blank=True, default=True, help_text='Sets the visibility for this link in KIOSK mode.', null=True),
        ),
    ]



# Generated by Django 3.0.11 on 2022-02-24 15:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0010_auto_20210929_1616'),
    ]

    operations = [
        migrations.AlterField(
            model_name='linkurl',
            name='url',
            field=models.URLField(blank=True, default='', help_text='redirect to external page', max_length=800),
        ),
    ]



import uuid

import django.http.request
from django.conf import settings
from django.contrib import admin, messages
from django.http import HttpResponse
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth import get_user_model
from django.template.response import TemplateResponse
from django.utils.safestring import mark_safe
from django.urls import path
from django.shortcuts import render, redirect, reverse

from .generator.forms import QrGenerateForm
from .generator.qr import create_qr_code
from .generator.zip import generate_qr_zip
from .models import ApiHit, Department, LinkUrl, QRCode, ApiCall, Header
from .serializers import ApiHitSerializer
from rest_framework.renderers import JSONRenderer

User = get_user_model()

admin.site.unregister(User)


class ApiHitQrCodeFilter(admin.RelatedFieldListFilter):
    title = 'QrCode'

    def __init__(self, field, request, params, model, model_admin, field_path):
        super().__init__(field, request, params, model, model_admin, field_path)
        if not request.user.is_superuser:
            self.lookup_choices = QRCode.objects.filter(department__in=request.user.departments.all()).values_list(
                'pk', 'title')


class ApiHitDepartmentFilter(admin.RelatedFieldListFilter):
    title = 'Department'

    def __init__(self, field, request, params, model, model_admin, field_path):
        super().__init__(field, request, params, model, model_admin, field_path)
        if not request.user.is_superuser:
            self.lookup_choices = request.user.departments.all().values_list('pk', 'name')


class DepartmentInline(admin.TabularInline):
    model = Department.users.through


@admin.register(User)
class MyUserAdmin(UserAdmin):
    # show_facets = admin.ShowFacets.NEVER
    inlines = [DepartmentInline]


@admin.register(ApiHit)
class ApiHitAdmin(admin.ModelAdmin):
    readonly_fields = ('hit_date', 'action', 'code', 'message')
    list_display = ('code', 'hit_date', 'action', 'message')
    change_list_template = 'qrtoolkit_core/apihit/change_list.html'
    # show_facets = admin.ShowFacets.NEVER

    # list_filter = ('code__department__name', 'code__title')

    def get_queryset(self, request):
        qs = super(ApiHitAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(code__department__in=request.user.departments.all())

    def get_list_filter(self, request):
        if request.method == 'GET' and request.GET.get('code__department__id__exact', None) is not None:
            return ('code__department', ApiHitDepartmentFilter), ('code', ApiHitQrCodeFilter)
        return ('code__department', ApiHitDepartmentFilter),

    def lookup_allowed(self, lookup, value):
        if lookup in ('code__department__id__exact', 'code__id__exact'):
            return True
        return super(ApiHitAdmin, self).lookup_allowed(lookup, value)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        cl = self.get_changelist_instance(request)

        hit_data = ApiHitSerializer(instance=cl.get_queryset(request), many=True).data
        extra_context['hits_json'] = JSONRenderer().render(hit_data).decode('utf-8')
        if request.method == 'GET' and request.GET.get('code__id__exact', None) is not None and request.GET.get(
                'code__department__id__exact', None) is None:
            nq = request.GET.copy()
            nq.pop('code__id__exact')
            request.GET = nq
            return super(ApiHitAdmin, self).changelist_view(request, extra_context=extra_context)
        return super(ApiHitAdmin, self).changelist_view(request, extra_context=extra_context)


class LinkUrlInline(admin.StackedInline):
    model = LinkUrl
    extra = 1


@admin.register(QRCode)
class QRCodeAdmin(admin.ModelAdmin):
    list_display = ('title', 'department',
                    'get_code_url', 'get_code_image_url', 'basic_info')
    list_filter = (('department', admin.RelatedOnlyFieldListFilter),)
    search_fields = ('title', 'department__name', 'basic_info')
    inlines = [LinkUrlInline]
    change_list_template = 'qrtoolkit_core/qrcode/change_list.html'
    actions = ['download_codes_action', ]
    readonly_fields = ('uuid',)
    # show_facets = admin.ShowFacets.NEVER

    def get_queryset(self, request):
        qs = super(QRCodeAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(department__in=request.user.departments.all())

    def get_model_perms(self, request):
        return super(QRCodeAdmin, self).get_model_perms(request)

    def get_code_image_url(self, obj):
        return mark_safe(
            f'<span><a href="/code/{obj.short_uuid}">/code/{obj.short_uuid}</a></span>')

    def get_code_url(self, obj):
        return mark_safe(
            f'<span><a href="{settings.REDIRECT_SERVICE_URL}/{obj.short_uuid}">/{obj.short_uuid}</a></span>')

    get_code_image_url.short_description = 'Code image'
    get_code_url.short_description = 'Code url'

    def download_codes_action(self, request, queryset):
        """

        :type queryset: django.db.models.query.QuerySet
        :type request: django.http.request.HttpRequest
        """
        pks = [str(x.pk) for x in queryset.all()]
        url = '%s' % (reverse('admin:admin_qr_download'),)
        request.session['codes_to_download'] = pks
        return redirect(url)

    download_codes_action.short_description = 'Download codes'

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'department' and not request.user.is_superuser:
            kwargs['queryset'] = request.user.departments.all()
        return super(QRCodeAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def get_changeform_initial_data(self, request):
        return {'department': request.user.departments.first()}

    def get_urls(self):
        urls = super(QRCodeAdmin, self).get_urls()
        custom_urls = [
            path('download/', self.admin_site.admin_view(self.download_codes), name='admin_qr_download'),
            path('scan/', self.admin_site.admin_view(self.scan_view), name='admin_scan_search')
        ]
        return custom_urls + urls

    def scan_view(self, request):
        if request.method == 'POST':
            short_uuid = request.POST['short_uuid']
            try:
                code = _get_qr_code(short_uuid)
                return redirect('admin:qrtoolkit_core_qrcode_change', object_id=code.pk)
            except QRCode.DoesNotExist:
                self.message_user(request, f'Qr code with uuid {short_uuid} not found.', level=messages.WARNING)

        context = dict(
            # Include common variables for rendering the admin template.
            self.admin_site.each_context(request),
            # Anything else you want in the context...
            is_nav_sidebar_enabled=False,
            title='Scan qr code'
        )
        return TemplateResponse(request, 'qrtoolkit_core/qrcode/scanner.html', context)

    def download_codes(self, request):
        """
        Downloads specified qr codes in a zip archive. qr code id's are provided by using the 'codes' query parameter

        :type request: django.http.HttpRequest
        """
        if request.method == 'POST':
            download = request.GET['download']
            pks = [int(x) for x in download.split(',')]
            if request.user.is_superuser:
                codes = QRCode.objects.filter(pk__in=pks).all()
            else:
                codes = QRCode.objects.filter(pk__in=pks, department__in=request.user.departments.all()).all()
            form = QrGenerateForm(data=request.POST)
            if form.is_valid():
                zf = generate_qr_zip(codes, form)
                resp = HttpResponse(zf, content_type="application/x-zip-compressed")
                resp[
                    'Content-Disposition'] = f'attachment; filename=generated_codes-{download.replace(",", "_")[:20]}.zip'
                return resp
            else:
                qrform = QrGenerateForm({
                    'dark': '#000000',
                    'light': '#FFFFFF',
                    'scale': '3',
                    'kind': 'svg'
                })
                qrform.is_valid()
                qr_image = create_qr_code(codes.first(), qrform).decode('utf-8')
                context = dict(
                    self.admin_site.each_context(request),
                    is_nav_sidebar_enabled=False,
                    title="Download Qr codes",
                    codes=codes,
                    form=form,
                    qr_image=qr_image
                )
                return render(request, 'qrtoolkit_core/qrcode/download.html', context=context)
        else:
            pks = request.session.get('codes_to_download')
            if request.user.is_superuser:
                codes = QRCode.objects.only('id', 'title').filter(pk__in=pks).all()
            else:
                codes = QRCode.objects.only('id', 'title').filter(pk__in=pks,
                                                                  department__in=request.user.departments.all()).all()
            qrform = QrGenerateForm({
                'dark': '#000000',
                'light': '#FFFFFF',
                'scale': '3',
                'kind': 'svg'
            })
            qrform.is_valid()
            qr_image = create_qr_code(codes.first(), qrform).decode('utf-8')
            context = dict(
                self.admin_site.each_context(request),
                is_nav_sidebar_enabled=False,
                title="Download Qr codes",
                codes=codes,
                form=QrGenerateForm(),
                qr_image=qr_image
            )

            return render(request, 'qrtoolkit_core/qrcode/download.html', context=context)


@admin.register(Department)
class DepartmentAdmin(admin.ModelAdmin):
    # show_facets = admin.ShowFacets.NEVER
    filter_horizontal = ('users',)


class HeaderInline(admin.TabularInline):
    model = Header
    extra = 1


@admin.register(ApiCall)
class ApiCallAdmin(admin.ModelAdmin):
    # show_facets = admin.ShowFacets.NEVER
    inlines = [HeaderInline]

    def _get_extra_context(self, request, extra_context=None):
        extra_context = dict() if extra_context is None else extra_context
        if not request.user.is_superuser:
            owner_codes = QRCode.objects.only('title').filter(department__in=request.user.departments.all(),
                                                              mode=QRCode.REDIRECT_MODE_CHOICES.API_CALL).all()
        else:
            owner_codes = QRCode.objects.only('title').filter(mode=QRCode.REDIRECT_MODE_CHOICES.API_CALL).all()
        extra_context['owner_codes'] = owner_codes
        return extra_context

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = self._get_extra_context(request, extra_context=extra_context)
        return super(ApiCallAdmin, self).change_view(request, object_id, form_url=form_url, extra_context=extra_context)

    def add_view(self, request, form_url='', extra_context=None):
        extra_context = self._get_extra_context(request, extra_context=extra_context)
        return super(ApiCallAdmin, self).add_view(request, form_url=form_url, extra_context=extra_context)

    def get_field_queryset(self, db, db_field, request):
        if db_field.name == 'link_url':
            if not request.user.is_superuser:
                return db_field.remote_field.model.objects.filter(code__department__in=request.user.departments.all(),
                                                                  code__mode=QRCode.REDIRECT_MODE_CHOICES.API_CALL)
            else:
                return db_field.remote_field.model.objects.filter(code__mode=QRCode.REDIRECT_MODE_CHOICES.API_CALL)
        return super(ApiCallAdmin, self).get_field_queryset(db, db_field, request)


admin.site.site_header = 'Qr Gent Administration'
admin.site.site_title = 'Qr Gent admin'
admin.site.site_url = settings.API_URL


def _get_qr_code(short_uuid):
    try:
        short_uuid = uuid.UUID(short_uuid)
        # short_uuid is a valid uuid object
        return QRCode.objects.get(uuid=short_uuid)
    except ValueError:
        # short_uuid is not a valid uuid object
        return QRCode.objects.get(short_uuid=short_uuid)



from django import forms
from django.conf import settings
from django.core import validators
from django.utils.text import get_valid_filename

QrKindTypes = [
    ('svg', 'Svg'),
    ('png', 'Png'),
    ('pdf', 'Pdf')
]

QrFilenameChoices = [
    ('title', 'title.ext'),
    ('envtitle', 'env-title.ext'),
    ('basic', 'basic_info.ext'),
    ('envbasic', 'env-basic_info.ext'),
    ('uuid', 'uuid.ext'),
    ('envuuid', 'env-uuid.ext')
]


class HexField(forms.CharField):
    def __init__(self, **kwargs):
        super(HexField, self).__init__(max_length=7, min_length=4, **kwargs)
        self.validators.append(validators.RegexValidator(regex='^#[0-9A-Fa-f]{3}(?:[0-9A-Fa-f]{3})?$'))


class QrGenerateForm(forms.Form):
    query = forms.HiddenInput()
    kind = forms.ChoiceField(choices=QrKindTypes, required=False, initial='svg')
    light = HexField(required=False, initial='#FFFFFF', widget=forms.TextInput(attrs={'type': 'color'}))
    dark = HexField(required=False, initial='#000000', widget=forms.TextInput(attrs={'type': 'color'}))
    scale = forms.IntegerField(required=False, initial=3)
    filename = forms.ChoiceField(choices=QrFilenameChoices, required=False, initial='title')

    def clean_kind(self):
        if not self['kind'].html_name in self.data:
            return self.fields['kind'].initial
        return self.cleaned_data['kind']

    def clean_light(self):
        if not self['light'].html_name in self.data:
            return self.fields['light'].initial
        return self.cleaned_data['light']

    def clean_dark(self):
        if not self['dark'].html_name in self.data:
            return self.fields['dark'].initial
        return self.cleaned_data['dark']

    def clean_scale(self):
        if not self['scale'].html_name in self.data:
            return self.fields['scale'].initial
        return self.cleaned_data['scale']

    def clean_filename(self):
        if not self['filename'].html_name in self.data:
            return self.fields['filename'].initial
        return self.cleaned_data['filename']

    def create_filename(self, code):
        filename_setting = self.cleaned_data['filename']
        ext = self.cleaned_data["kind"]
        name = f'{code.title}.{ext}'
        if filename_setting == 'envtitle':
            name = f'{settings.ENVIRONMENT}-{code.title}.{ext}'
        if filename_setting == 'basic':
            subname = code.basic_info[:200].replace(':', '_')
            name = f'{subname}.{ext}'
        if filename_setting == 'envbasic':
            subname = code.basic_info[:200].replace(':', '_')
            name = f'{settings.ENVIRONMENT}-{subname}.{ext}'
        if filename_setting == 'uuid':
            name = f'{code.short_uuid}.{ext}'
        if filename_setting == 'envuuid':
            name = f'{settings.ENVIRONMENT}-{code.short_uuid}.{ext}'

        return get_valid_filename(name)



from django.core.exceptions import ValidationError
from django.test import TestCase
from qrtoolkit_core.models import QRCode, LinkUrl
from qrtoolkit_core.validators import validate_is_api_mode_qrcode
from string import ascii_letters, digits, punctuation
from random import choice


def generate_text(amount_chars):
    chars_to_pick = ascii_letters + digits + punctuation
    return ''.join([choice(chars_to_pick) for _ in range(amount_chars)])


class QrCodeTestCase(TestCase):
    def setUp(self):
        QRCode.objects.create(title='qr1')

    def test_can_add_basic_info_to_code(self):
        qr = QRCode.objects.get(title__exact='qr1')
        info = generate_text(20)
        qr.basic_info = info
        qr.save()
        qr = QRCode.objects.get(title__exact='qr1')
        self.assertEqual(qr.basic_info, info)

    def test_create_link_url_adds_to_set(self):
        qr = QRCode.objects.get(title__exact='qr1')
        link = LinkUrl.objects.create(url='https://www.stad.gent/', code_id=qr.id, name='stad gent')
        link.save()

        self.assertEqual(qr.urls.count(), 1)
        self.assertEqual(qr.urls.first(), link)

    def test_urls_are_ordered_by_priority(self):
        qr = QRCode.objects.get(title__exact='qr1')
        self.assertEqual(qr.urls.count(), 0)
        link = LinkUrl.objects.create(url='https://stad.gent/1/', code=qr, name='stad gent 1', priority=1.0)
        link2 = LinkUrl.objects.create(url='https://stad.gent/2/', code=qr, name='stad gent 2', priority=2.0)
        link.save()
        link2.save()
        link_set = qr.urls
        self.assertEqual(link_set.count(), 2)
        self.assertEqual(link_set.first(), link2)

    def test_validator_qr_mode_api_call_works_correctly(self):
        qr = QRCode.objects.get(title__exact='qr1')
        qr.mode = QRCode.REDIRECT_MODE_CHOICES.KIOSK
        qr.save()
        link = LinkUrl.objects.create(url='https://stad.gent/1/', code=qr, name='stad gent 1', priority=1.0)
        link.save()
        self.assertRaises(ValidationError, lambda: validate_is_api_mode_qrcode(link.id))

        qr.mode = QRCode.REDIRECT_MODE_CHOICES.API_CALL
        qr.save()
        try:
            validate_is_api_mode_qrcode(link.id)
        except ValidationError:
            self.fail(
                'validate_is_api_mode_qrcode should not throw if a QrCode,'
                ' linked to a specific LinkUrl is in API_CALL mode')
